/**
 * \file msrun_classi.cpp
 * \date 18 sept. 2009
 * \author Olivier Langella
 */

#include "msrun_classic.h"
#include "../share/utilities.h"
#include "../xic/xic_base.h"


#include <math.h>


MsrunClassic::MsrunClassic(const QString & msrunid)
    :
    Msrun(msrunid)
{
}

MsrunClassic::~MsrunClassic() {
    this->deleteSpectra();
}

/// delete the spectra of this msrun, we do not need it anymore
void
MsrunClassic::deleteSpectra() {
    std::map<mcq_double, spectrum *>::iterator it;
    for (it = _map_lc.begin(); it != _map_lc.end(); ++it) {
        if (it->second != 0) {
            delete (it->second);
            (it->second) = 0;
        }
    }
    _map_lc.clear();
}

/// set spectrum (parsed from the mzxml file)
void
MsrunClassic::setSpectrum(const mcq_double retention_time, spectrum * the_spectrum) {
    _map_lc[retention_time] = the_spectrum;
}

void
MsrunClassic::prepareSpectraForQuantification(const XicExtractionMethodBase *
        xic_extraction_method) {
    this->parseSpectra();
}

void
MsrunClassic::doneWithSpectra() {
    deleteSpectra();
}

McqXicNoConstSp
MsrunClassic::extractXic(McqXicType xic_type,const pappso::MassRange & look_for_mz) const {
    /// extract XIC
    McqXicNoConstSp ptr_on_xic = std::make_shared<xicBase>(xicBase(this, look_for_mz, xic_type));
    setXicVectors(ptr_on_xic);
    if(ptr_on_xic.get()->size() == 0) {
        ptr_on_xic = nullptr;
    }
    return (ptr_on_xic);
}

void
MsrunClassic::setXicVectors(McqXicNoConstSp ptr_on_xic) const {
    std::map<mcq_double, spectrum *>::const_iterator it_map_lc;
    for (it_map_lc = this->begin_map_lc();
            it_map_lc != this->end_map_lc();
            ++it_map_lc) {
        mcq_double sum_or_max = ptr_on_xic.get()->getIntensityFromSpectrum(*(it_map_lc->second));
        ptr_on_xic.get()->push_back(pappso::XicElement(it_map_lc->first, sum_or_max));
    }
}
