/**
 * \file msrun_sliced.cpp
 * \date June 14, 2011
 * \author Edlira Nano
 */


#include "msrun_sliced.h"
#include "../mass_chroq.h"
#include "../share/utilities.h"
#include "../xic/xic_base.h"
#include "../quanti_items/quantiItemBase.h"


MsrunSliced::MsrunSliced(const QString & msrunid, const int nb_of_slices) 
  :
  Msrun(msrunid),
  _nb_of_slices(nb_of_slices)
{
  _slicer = 0;
}

MsrunSliced::~MsrunSliced() {
  deleteSlicer();
}

void
MsrunSliced::deleteSlicer() {
  if (_slicer != 0) {  
    delete (_slicer);
    _slicer = 0;
  }
}
  

void 
MsrunSliced::setSpectrum(const mcq_double retention_time, spectrum * the_spectrum) {
  _slicer->setSpectrumInSlices(retention_time, the_spectrum);
  /// very important to delete the old spectrum here, we do not need it anymore
  if (the_spectrum != 0) {
    delete (the_spectrum);   
    the_spectrum = 0; 
  }
}

void
MsrunSliced::prepareSpectraForQuantification(const XicExtractionMethodBase * 
					     xic_extraction_method) {
    this->setSlicer(xic_extraction_method);
    this->parseSpectra();
    _slicer->closeCacheInSlices();
}

void
MsrunSliced::doneWithSpectra() {
  _slicer->clearSlices();
  deleteSlicer();
 }
 
// set the slicing for this msrun 
void
MsrunSliced::setSlicer(const XicExtractionMethodBase * xic_extraction_method) {
  deleteSlicer();
  const mcq_double min_mz = this->get_low_mz();
  const mcq_double max_mz = this->get_high_mz();
  mcq_double overlap_range;
  if (xic_extraction_method != NULL) {
    pappso::MassRange range(max_mz, xic_extraction_method->getPappsoPrecision());
    overlap_range = range.getHighest() - range.getLowest();
  } else {
    overlap_range = 5;
  }
  qDebug() << "initializing slicer with _nb_of_slices, low_mz, high_mz " << _nb_of_slices
	   << ", " << min_mz << ", " << max_mz;
  _slicer = new Slicer(_nb_of_slices, min_mz, max_mz, overlap_range);
}

McqXicNoConstSp
MsrunSliced::extractXic(McqXicType xic_type, const pappso::MassRange & mz_range) const {
  /// wake the right slicer
  mcq_slice_index_id slice_index = _slicer->wakeUpSliceForMz(mz_range);
  
  /// extract XIC
  McqXicNoConstSp ptr_on_xic = std::make_shared<xicBase>(xicBase(this, mz_range, xic_type));
  
  /// setXicVectors    
  _slicer->setXicVectors(ptr_on_xic);
    
  
  _slicer->releaseSlice(slice_index);
  if(ptr_on_xic.get()->size() == 0) {
    ptr_on_xic = nullptr;
  }
  return (ptr_on_xic);
}
