
/*******************************************************************************
 * Copyright (c) 2016 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of the MassChroQ software.
 *
 *     MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/
#include "mapquanti.h"
#include "../quanti_items/quantiItemBase.h"
#include "../xic/xicfilterdetectmatch.h"
#include "../xic/xicfilterdetectmatchrealrt.h"
#include "../xic/xic_base.h"
#include "../xic/xictracebase.h"
#include "../consoleout.h"
#include <cmath>

MapQuanti::MapQuanti(MonitorSpeedInterface & monitor, Msrun * current_msrun, McqXicType xic_type, Quantificator * quantificator,
                     const pappso::XicFilterInterface * p_xic_filter,const pappso::XicDetectionInterface * p_detection)
    :_monitor(monitor), _xic_type(xic_type) {
    _p_msrun = current_msrun;
    _p_xic_filter = p_xic_filter;
    _p_detection = p_detection;
    _matching_mode = quantificator->getMatchingMode();
    _quantificator = quantificator;
}

void MapQuanti::operator()(QuantiItemBase * p_currentSearchItem)
{
    qDebug() << " MapQuanti::operator begin";
    try {
        pappso::pappso_double rt_target = -1;
        //std::vector<MsMsRtIntensity> rt_list;

        PeptideRt * p_peptide_rt = p_currentSearchItem->getPeptideRtSp().get();

        if (p_peptide_rt != nullptr) {
            rt_target = p_peptide_rt->getRtTarget(_p_msrun, _matching_mode);

            if (std::isnan(rt_target)) {
                qDebug() << " MapQuanti::operator() isnan(rt_target) " << p_currentSearchItem->getQuantiItemId() << " " << rt_target;
                throw mcqError(QObject::tr("Error in MapQuanti::operator() isnan(rt_target) %1")
                               .arg(p_currentSearchItem->getQuantiItemId()));
            }
        }

        if (rt_target == -1) {
            if (_matching_mode == McqMatchingMode::post_matching) {
                //it's Ok, just detect and keep peaks
            } else if (_matching_mode == McqMatchingMode::no_matching) {
                rt_target = p_currentSearchItem->getRt();
            } else {
                qDebug() << " MapQuanti::operator rt_target == -1 exit";
                return;
            }
        }
        qDebug() << " MapQuanti::operator _p_msrun->extractXic";
        McqXicNoConstSp _p_xic = _p_msrun->extractXic(_xic_type, _quantificator->getXicExtractionMethod()->getPappsoMassRange( p_currentSearchItem->getMz()));

        if (_p_xic == nullptr) {
            qDebug() << "detectAndQuantify null xic ";

            qDebug() << " MapQuanti::operator begin 2";
        }
        else {
            qDebug() << " MapQuanti::operator begin 3 rt_target=" << rt_target;
            XicTraceBase * p_xic_trace = nullptr;

            //only take an rt reference to match peak
            p_xic_trace = p_currentSearchItem->newXicTrace(_quantificator, _p_msrun, "target");


            XicFilterDetectMatch filter_detect_match(p_xic_trace, _monitor, _p_xic_filter, _p_detection, _matching_mode) ;
            filter_detect_match.filterDetectQuantify(*(_p_xic.get()), rt_target, p_currentSearchItem, _p_msrun);


            if (p_xic_trace != nullptr) {
                p_xic_trace->setAlignedRtTarget(_p_msrun->getAlignedRtByOriginalRt(rt_target));
                p_xic_trace->setRtTarget(rt_target);
                p_xic_trace->setXic(_p_xic);
                p_xic_trace->close();
                delete p_xic_trace;
            }
        }
        qDebug() << " MapQuanti::operator end";
    }
    catch (mcqError& errorException)
    {
        // QObject::tr("%1 CPUs used").arg(cpu_number)
        // mcqerr() << "Oops! an error occurred in MassChroQ. Dont Panic :" << endl;
        // mcqerr() << error.qwhat() << endl;
        qDebug() << " MapQuanti::operator " << errorException.qwhat();
        throw mcqError(QObject::tr("Error in MapQuanti::operator() :\n %1")
                       .arg(errorException.qwhat()));
        // windaube_exit();
        // return 1;
    }
}
