/**
* \file mzxmlSimpleParser.cpp
* \date May, 22 2011
* \author Olivier Langella, Edlira Nano
*/

// test /gorgone/pappso/moulon/raw/20130219_Amaizing_plaque_1/20130219_blein_amaizing_1_A02_2.mzXML
// test /gorgone/pappso/moulon/raw/20140219_Amaizing_plaque_12/20130711_amaizing_blein_1_E07_991.mzXML
// error in msrun 'samp1001' (in getPrecursorIntensity method): scan number 2068 does not exist
#include "mzxmlSimpleParser.h"
#include <iterator>


MzxmlSimpleParser::MzxmlSimpleParser(Msrun * ms_run)  :
    XmlSimpleParser(ms_run)
{

}

MzxmlSimpleParser::~MzxmlSimpleParser() {
}


bool
MzxmlSimpleParser::startElement(const QString & /* namespaceURI */,
                                const QString & /* localName */,
                                const QString & qName,
                                const QXmlAttributes & attributes) {

    bool is_ok = true;
    if (qName == "scan") {
        is_ok = startElement_scan(attributes);
    }

    else if (qName == "precursorMz") {
        is_ok = startElement_precursorMz(attributes);
    }

    _currentText.clear();
    return is_ok;
}

bool
MzxmlSimpleParser::startElement_scan(const QXmlAttributes & attributes) {
    _current_scan._precursor_intensity = 0;
    _current_scan._precursor_mz = 0;
    _current_scan._ms_level=0;
    _current_scan._number=0;
    _current_scan._precursor_scan_num=0;
    _current_scan._retention_time=0;

    QString scan_number = attributes.value("num");
    QString ms_level = attributes.value("msLevel");

    if (scan_number.isEmpty()) {
        _errorStr = QObject::tr("the scan number attribute in mzXML file must not be empty\n");
        return false;
    }
    if (ms_level.isEmpty()) {
        _errorStr = QObject::tr("the msLevel attribute in mzXML file must not be empty\n");
        return false;
    }

    _current_scan_number = scan_number.toInt();
    _stack_scan_num.push_back(_current_scan_number);
    _ms_level = ms_level.toInt();

    _current_scan._number = _current_scan_number;
    _current_scan._ms_level = _ms_level;

    /// for all ms-levels
    QString temp = attributes.value("retentionTime").mid
                   (2, (attributes.value("retentionTime").size() - 3));
    _retention_time = temp.toDouble();

    _current_scan._retention_time = _retention_time;

    if ( _ms_level == 1 ) {
        QString lowMz = attributes.value("lowMz");
        QString highMz = attributes.value("highMz");
        if ( lowMz.isEmpty() || highMz.isEmpty() ) {
            _scan_type = "Full";
        } else {
            _low_mz = lowMz.toDouble();
            _high_mz = highMz.toDouble();
            this->set_low_mz();
            this->set_high_mz();
            if ( (_high_mz - _low_mz) > 100 )
                _scan_type = "Full";
            else
                _scan_type = "Zoom";
        }
    }

    // for ms_levels >= 2
    if ( _ms_level >= 2 ) {
        _precursor_scan_num = _current_scan_number;

        QString temp = attributes.value("retentionTime").mid
                       (2, (attributes.value("retentionTime").size() - 3));
        _precursor_rt = temp.toDouble();

        if (_ms_level == 2)
            _tic_level2 = (attributes.value("totIonCurrent")).toDouble();
    }
    _map_num2scan.insert(std::pair<int,scan_struct>(_current_scan._number,_current_scan));
    return true;
}

bool
MzxmlSimpleParser::startElement_precursorMz(const QXmlAttributes & attributes) {
    /// only for ms-level >= 2
    //if (_ms_level == 2) {
    std::list<int>::reverse_iterator rit = _stack_scan_num.rbegin()++;
    if (rit == _stack_scan_num.rend()) {
        _errorStr = QObject::tr("error in parser : the stack is empty\n");
        return false;
    }
    std::map<int , scan_struct >::iterator mapIt = _map_num2scan.find(*rit);
    if (mapIt == _map_num2scan.end()) {
        _errorStr = QObject::tr("error in parser : scan number %1 not found\n").arg(*rit);
        return false;
    }
    scan_struct & current_scan = mapIt->second;
    //if we have a "precursorScanNum" attribute, then MS levels are not written inside an other but uses this number as a reference !!!
    // so _precursor_scan_num and _precursor_rt are not the current values
    QString precursorScanNum = attributes.value("precursorScanNum");
    current_scan._precursor_scan_num = 0;

    if (precursorScanNum.isEmpty()) {
        //take n-1 scan number :
        //.rbegin()++;
        rit++;
        if (rit == _stack_scan_num.rend()) {
            _errorStr = QObject::tr("unable to find the parent scan number of the current scan number : %1\n").arg(current_scan._number);
            return false;
        }
        current_scan._precursor_scan_num = *rit;
    }
    else {
        //we must have a map of scan and the "_precursor_rt" from it
        current_scan._precursor_scan_num = precursorScanNum.toInt();
    }


    QString prec_int = attributes.value("precursorIntensity");
    if (prec_int.isEmpty() ||
            prec_int.toDouble() == 0) {
        _precursor_intensity_level2 = _tic_level2;
    } else {
        _precursor_intensity_level2 = prec_int.toDouble();
    }
    current_scan._precursor_intensity = _precursor_intensity_level2;
    // }
    return true;
}

bool MzxmlSimpleParser::set_precursor(scan_struct & scan) {
    qDebug() << "MzxmlSimpleParser::set_precursor(scan_struct & scan) begin";
    qDebug() << "scan num : " << scan._number;
    qDebug() << "scan level : " << scan._ms_level;
    qDebug() << "precursor scan : " << scan._precursor_scan_num;
    mcq_double precursor_intensity = scan._precursor_intensity;
    mcq_double precursor_mz = scan._precursor_mz;
    mcq_double rt = scan._retention_time;
    std::map<int , scan_struct >::iterator mapIt;
    scan_struct & scanLevel2 = scan;
    if (scan._ms_level > 2) {
        /* in ms-levels >= 2 we have a new tag : precursorMz
        witch has an attribute precursorIntensity. This attribute is
        set to 0 in ms-level >= 3 and in those levels
        we have to take the one corresponding to the corresponding
        ms-level 2*/

        //=> move to level 2 :
        while (scanLevel2._ms_level>2) {
            mapIt = _map_num2scan.find(scanLevel2._precursor_scan_num);
            if (mapIt == _map_num2scan.end()) {
                _errorStr = QObject::tr("error in parser : scan number %1 not found (from child scan number %2)\n").arg(scanLevel2._precursor_scan_num).arg(scanLevel2._number);
                return false;
            }
            scanLevel2= mapIt->second;
        }
        precursor_intensity = scanLevel2._precursor_intensity;
        precursor_mz = scanLevel2._precursor_mz;
        qDebug() << "scan._ms_level > 2";
        qDebug() << "scan num : " << scanLevel2._number;
        qDebug() << "scan level : " << scanLevel2._ms_level;
        qDebug() << "precursor scan : " << scanLevel2._precursor_scan_num;
    }
    return setPrecursor(scan._number, rt,
                        precursor_intensity, precursor_mz);

    /*
    //find the precursor_rt :
    mapIt = _map_num2scan.find(scanLevel2._precursor_scan_num);
    if (mapIt == _map_num2scan.end()) {
        _errorStr = QObject::tr("error in parser : scan number %1 not found (MS level 1 expected)\n").arg(scanLevel2._precursor_scan_num);
        return false;
    }
    if (mapIt->second._ms_level != 1) {
        _errorStr = QObject::tr("error in parser : MS level 1 expected for scan number %1\n").arg(mapIt->second._number);
        return false;
    }

    return setPrecursor(scan._number, mapIt->second._retention_time,
                        precursor_intensity, precursor_mz);

    */
}

bool
MzxmlSimpleParser::endElement(const QString & /* namespaceURI */,
                              const QString & /* localName */,
                              const QString & qName) {
    bool is_ok = true;

    if (qName == "precursorMz") {
        std::list<int>::reverse_iterator rit = _stack_scan_num.rbegin()++;
        if (rit == _stack_scan_num.rend()) {
            _errorStr = QObject::tr("error in parser : the stack is empty\n");
            return false;
        }
        std::map<int , scan_struct >::iterator mapIt = _map_num2scan.find(*rit);
        if (mapIt == _map_num2scan.end()) {
            _errorStr = QObject::tr("error in parser : scan number %1 not found\n").arg(*rit);
            return false;
        }
        scan_struct & current_scan = mapIt->second;

        _precursor_mz = _currentText.toDouble();
        current_scan._precursor_mz = _precursor_mz;

        is_ok = this->set_precursor(current_scan);
    }
    else if (qName == "peaks" && _ms_level == 1 && _scan_type == "Full") {
        _msrun->addOriginalRetentionTime(_retention_time);
    }
    else if (qName == "scan") {
        _stack_scan_num.pop_back();
    }
    _currentText.clear();
    return is_ok;
}

